@@ -11,7 +11,7 @@ class PublicationPlot(Plot):
         self.stretch(top=-0.02, left=0.15, right=0.05, bottom=0.1)
         self._outset_bbox_to_anchor = {
             'upper center': (0.5, 1.32),
-            'lower center': (0.5, -0.43),
+            'lower center': (0.5, -0.45),
             'upper right': (1.7, 1.035),
             'lower right': (1.7, -0.035),
         }
@@ -39,6 +39,9 @@ class LowResPlot(Plot):
             'lower right': (1.35, -0.035),
         }
 
+    def set_title(self, title, loc='left', x=0.12, y=0.98, text_obj=None):
+        super(LowResPlot, self).set_title(title, loc=loc, x=x, y=y, text_obj=text_obj)
+
     def set_subtitle(self, title, loc='right', x=0.9, y=0.92):
         super(LowResPlot, self).set_subtitle(title, loc=loc, x=x, y=y)
 
