@@ -1,20 +1,27 @@
 #!/usr/bin/env python3
 
 import os
+import subprocess
 import numpy as np
 import copy
 import statistics as stats
+import colorsys
+import warnings
 import matplotlib as mpl
 import matplotlib.pyplot as plt
+import matplotlib.colors as mc
+
 from mpl_toolkits.mplot3d import Axes3D #required for 3D plots
 from matplotlib import image as mpimg
 from matplotlib.patches import Circle, Rectangle, Arrow, FancyBboxPatch, BoxStyle
 from matplotlib import patheffects
 
+from distutils.spawn import find_executable
 from tempfile import gettempdir
 from collections import Iterable
 from itertools import cycle
 from time import time
+from subprocess import Popen
 
 
 try:
@@ -36,8 +43,17 @@ Maureen = {
     'purple': MAUREENSTONE_COLORS[5],
     'cardinal': MAUREENSTONE_COLORS[6],
     'gold': MAUREENSTONE_COLORS[7],
-
 }
+Vibrant = {
+    'cyan': '#33BBEE',
+    'magenta': '#EE3377',
+    'teal': '#009988',
+    'orange': '#EE7733',
+    'blue': '#0077BB',
+    'red': '#CC3311',
+    'gray': '#BBBBBB',
+}
+MARKERS = ['o', 'X', 'v', '*', 'd', 's', '*', 'p']
 LIGHT_GRAY = '#D3D3D3'
 PALETTE_NAME = MAUREENSTONE_COLORS
 COLORMAP_3D = 'YlGnBu'
@@ -51,6 +67,42 @@ mpl.style.use('seaborn-poster')
 mpl.rcParams['mathtext.fontset'] = 'cm'  # Font for tex
 
 
+def usetex(use=True, silent=False, force=False):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        if not use:
+            mpl.rc('text', usetex=False)
+        elif find_executable('latex'):
+            if force or Popen(
+                ['kpsewhich', 'type1ec.sty'],
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL).communicate()[0]:
+                    mpl.rc('text', usetex=True)
+        elif not silent:
+            print('texlive-full is not installed, plotify cannot use tex.')
+
+
+def lighten_color(color, amount=0.5):
+    """
+    Taken from:
+    https://stackoverflow.com/questions/37765197/darken-or-lighten-a-color-in-matplotlib/49601444
+
+    Lightens the given color by multiplying (1-luminosity) by the given amount.
+    Input can be matplotlib color string, hex string, or RGB tuple.
+
+    Examples:
+    >> lighten_color('g', 0.3)
+    >> lighten_color('#F034A3', 0.6)
+    >> lighten_color((.3,.55,.1), 0.5)
+    """
+    try:
+        c = mc.cnames[color]
+    except:
+        c = color
+    c = colorsys.rgb_to_hls(*mc.to_rgb(c))
+    return colorsys.hls_to_rgb(c[0], max(0, min(1, amount * c[1])), c[2])
+
+
 def set_box_color(canvas, bp, color):
     artists = bp['boxes'] + bp['whiskers'] + bp['caps']
     for a in artists:
@@ -62,6 +114,7 @@ def set_box_color(canvas, bp, color):
 class Plot(object):
 
     def __init__(self, title='', height=3900.0, width=7200.0, dpi=600.0, plot3d=False, border=True):
+        usetex(True, silent=True)
         self.dpi = float(dpi)
         self.figure = plt.figure(figsize=(width/dpi, height/dpi), dpi=dpi)
         self.height = height
@@ -88,6 +141,7 @@ class Plot(object):
             'upper right': (1.25, 1.025),
             'lower right': (1.25, -0.025),
         }
+        self.markers = cycle(MARKERS)
 
     def _preprint(self):
         handles, labels = self.canvas.get_legend_handles_labels()
@@ -159,6 +213,11 @@ class Plot(object):
         self.figure.set_figwidth(w, forward=True)
 
     def save(self, path, **kwargs):
+        # create path if non-existant
+        plot_dir = os.path.dirname(path)
+        os.makedirs(plot_dir, exist_ok=True)
+
+        # save figure
         if path[-4:] == 'html':
             try:
                 from plotly.tools import mpl_to_plotly
@@ -240,7 +299,24 @@ class Plot(object):
         color = kwargs.pop('color', None)
         if color is None:
             color = next(self.colors)
-        self.canvas.plot(x, y, color=color, *args, **kwargs)
+        marker = kwargs.pop('marker', None)
+        if marker is None:
+            marker = next(self.markers)
+        elif marker is False:
+            marker = None
+        markevery = kwargs.pop('markevery', None)
+        if markevery is None:
+            markevery = 1 if len(x) < 20 else len(x) // 10
+        markersize = kwargs.pop('markersize', 8.5)
+        self.canvas.plot(
+            x,
+            y,
+            color=color,
+            marker=marker,
+            markevery=markevery,
+            markersize=markersize,
+            *args,
+            **kwargs)
         if isinstance(jitter, list):
             top = [v + j for v, j in zip(y, jitter)]
             low = [v - j for v, j in zip(y, jitter)]
@@ -252,15 +328,17 @@ class Plot(object):
             low = y - jitter
             self.canvas.fill_between(x, low, top, alpha=0.15, linewidth=0.0, color=color)
 
-    def bar(self, x, y=None, *args, **kwargs):
+    def bar(self, x, y=None, show_values=False, *args, **kwargs):
         # TODO: Show variance and mean on bars (as an option)
         if y is None:
             y = x
             x = list(range(len(y)))
-        fake_x = list(range(len(x)))
-        for i in fake_x:
+        fake_x = list(range(1, 1 + len(x)))
+        used_colors = []
+        for i in range(len(fake_x)):
             color = next(self.colors)
-            self.canvas.bar(x=i, height=y[i], color=color, *args, **kwargs)
+            self.canvas.bar(x=fake_x[i], height=y[i], color=color, *args, **kwargs)
+            used_colors.append(color)
         self.canvas.set_xticks(fake_x)
         self.canvas.set_xticklabels(x)
         plt.setp(self.canvas.get_xticklabels(),
@@ -268,6 +346,21 @@ class Plot(object):
                  ha='right',
                  rotation_mode='anchor')
 
+        if show_values:
+            y_means = y
+            x_means = fake_x
+            # Reversed because we want the latest plotted ticks.
+            # (e.g. when plotted one at a time.)
+            for x_m, y_m, c_m in zip(reversed(x_means), reversed(y_means), reversed(used_colors)):
+                text = self.canvas.text(x_m, y_m, '%.2f' % y_m, color=c_m,
+                                        horizontalalignment='center',
+                                        verticalalignment='bottom',
+                                        fontweight='bold')
+                text.set_path_effects([patheffects.withStroke(linewidth=0.7, foreground=c_m)])
+        return fake_x
+
+
+
     def box(self, x, y, show_values=False, num_box_sets=None, spacing=2.0, center_ticks=False, *args, **kwargs):
         # Arguments and Defaults
         if num_box_sets is not None:
@@ -283,7 +376,10 @@ class Plot(object):
         assert len(x) == len(y), 'x, y not same length'
 
         # Boxes
-        positions = np.array(range(len(x))) * (spacing * self._box_num_sets) + self._box_curr_set * 0.8
+        positions = kwargs.pop('positions', None)
+        if positions is None:
+            positions = np.array(range(len(x))) * (spacing * self._box_num_sets) \
+                      + self._box_curr_set * 0.8
         boxplot = self.canvas.boxplot(y,
                                       positions=positions,
                                       sym='',
@@ -309,8 +405,8 @@ class Plot(object):
         # Ticks formatting
         if center_ticks:
             mid_positions = np.array(range(len(x))) * (spacing * self._box_num_sets) + (self._box_num_sets - 1) * 0.4
-            self.canvas.set_xticks([], False)
-            self.canvas.set_xticks(mid_positions, False)
+            self.canvas.set_xticks(ticks=[], minor=False)
+            self.canvas.set_xticks(ticks=mid_positions, minor=False)
             self.canvas.set_xticklabels(x)
         plt.setp(self.canvas.get_xticklabels(),
                  rotation=35,
@@ -327,7 +423,12 @@ class Plot(object):
         color = kwargs.pop('color', None)
         if color is None:
             color = next(self.colors)
-        self.canvas.scatter(color=color, *args, **kwargs)
+        marker = kwargs.pop('marker', None)
+        if marker is None:
+            marker = next(self.markers)
+        elif marker is False:
+            marker = None
+        self.canvas.scatter(color=color, marker=marker, *args, **kwargs)
 
     def heatmap(self, heatvalues, xlabels=None, ylabels=None, show_values=False, cbar_title='', *args, **kwargs):
         '''
@@ -515,6 +616,10 @@ class Plot(object):
         self.palette = palette
         if 'maureen' in palette or 'custom' in palette:
             palette = MAUREENSTONE_COLORS
+        elif palette == 'vibrant':
+            vibrant_order = ['cyan', 'magenta', 'orange',
+                             'teal', 'blue', 'red', 'gray']
+            palette = [Vibrant[vo] for vo in vibrant_order]
         elif isinstance(palette, str):
             cmap = mpl.cm.get_cmap(palette)
             palette = [cmap(i) for i in np.linspace(0.1, 1.0, num_colors)]
@@ -616,7 +721,7 @@ class Plot(object):
             self.canvas.xaxis.grid(False)
             self.canvas.yaxis.grid(False)
 
-    def set_legend(self, loc='best', show=True, inset=True, ncol=1, alpha=0.8, **kwargs):
+    def set_legend(self, loc='best', title=None, show=True, inset=True, ncol=1, alpha=0.8, **kwargs):
         # Here process the position
         legend_options = {}
         legend_location = loc
@@ -630,6 +735,7 @@ class Plot(object):
             'visible': show,
             'ncol': ncol,
             'framealpha': alpha,
+            'title': title,
         }
         legend_options.update(kwargs)
         self._legend_options = legend_options
@@ -657,6 +763,9 @@ class Plot(object):
     def close(self):
         plt.close(self.figure)
 
+    def usetex(self, *args, **kwargs):
+        usetex(*args, **kwargs)
+
 
 class Drawing(Plot):
 
@@ -703,8 +812,12 @@ class Plot3D(Plot):
     def surface(self, x, y, z=None, alpha=0.25, linewidth=0, *args, **kwargs):
         X, Y, Z = self._3d_preprocess(x, y, z)
         self.canvas.plot_surface(X=X, Y=Y, Z=Z, rstride=1, cstride=1,
-                cmap=self.colormap, linewidth=linewidth, antialiased=True, alpha=alpha,
-                *args, **kwargs)
+                                 cmap=self.colormap,
+                                 linewidth=linewidth,
+                                 antialiased=True,
+                                 alpha=alpha,
+                                 *args,
+                                 **kwargs)
 
     def wireframe(self, x, y, z=None, *args, **kwargs):
         X, Y, Z = self._3d_preprocess(x, y, z)
@@ -723,6 +836,9 @@ class Plot3D(Plot):
         self.canvas.contour(X, Y, Z, zdir='y', offset=ymin, cmap=self.colormap)
 
     def set_camera(self, elev=None, azim=None):
+        """
+        Both parameters are angles in [0, 360].
+        """
         self.canvas.view_init(elev, azim)
 
     def set_notation(self, x='decimal', y='decimal', z='decimal'):
@@ -743,9 +859,9 @@ class Plot3D(Plot):
         self.canvas.ticklabel_format(style='sci', axis='z', scilimits=(-zra, zra))
 
     def set_axis(self, xtitle='', ytitle='', ztitle='', notation='scientific'):
-        self.canvas.set_xlabel(xtitle, labelpad=12)
-        self.canvas.set_ylabel(ytitle, labelpad=12)
-        self.canvas.set_zlabel(ztitle, labelpad=12)
+        self.canvas.set_xlabel(xtitle, labelpad=25)
+        self.canvas.set_ylabel(ytitle, labelpad=25)
+        self.canvas.set_zlabel(ztitle, labelpad=25)
 
 
 class Container(Plot):
